#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "sensor8.h"
#include "i2c.h"
#include "mydebug.h"

// Minimum delay is 4.7 microseconds, MAXIMUM speed of sensor20 is 50 microseconds.
// No delay needed, so this becomes a do-nothing stub.
#define I2C_delay() { }

// TODO:  make this timeout
#define CLOCKSTRETCH()  while (pin_read(FD, SCL) == 0);  // Clock stretching

static int started = 0;

static void pin_clear(int FD, int pin)	{	write(FD, set_lo+pin, 1);	}
static int pin_read(int FD, int pin)
{
	unsigned char bit=0;
	write(FD, digital_in+pin, 1);
	read(FD, &bit, 1);
	return(bit);
}

static void arbitration_lost(void) { fprintf(stderr, "wryyyyyyyyyyyyy"); exit(1); }
static unsigned char i2c_read_byte(int fd, int SCL, int SDA, int nack, int send_stop);
static int i2c_write_byte(int fd, int SCL, int SDA, int send_start, int send_stop, unsigned char byte);

int cmd8_i2c_write7_multi(int fd, int scl, int sda, int devid, int reg, 
	const unsigned char *buf, int len)
{
	if((scl < 0)||(scl>7)||(sda<0)||(sda>7)) return(-1);

	if(i2c_write_byte(fd,scl,sda,1,0,(devid<<1)|0) != 0)
		return(-1);
	if(i2c_write_byte(fd,scl,sda,0, 0, reg) != 0)
		return(-1);

	while(len > 1)
	{
		if(i2c_write_byte(fd, scl, sda, 0, 0,  *(buf++)) != 0)
			return(-1);

		len--;
	}

	if(i2c_write_byte(fd,scl,sda,0, 1, *buf) != 0)
		return(-1);

	return(0);
}

int cmd8_i2c_read7_multi(int fd, int scl, int sda, int devid, int reg, int len)
{
	if((scl < 0)||(scl>7)||(sda<0)||(sda>7)) return(-1);

	if(i2c_write_byte(fd,scl,sda,1,0,(devid<<1)|0) != 0)
		return(-1);
	if(i2c_write_byte(fd,scl,sda,0, 0, reg) != 0)
		return(-1);
	if(i2c_write_byte(fd,scl,sda,1, 0, (devid<<1)|1) != 0)
		return(-1);

	while(len > 1)
	{
		printf("%02x", i2c_read_byte(fd, scl, sda, 0, 0));
		len--;
	}

	printf("%02x\n", i2c_read_byte(fd,scl,sda,1,1));
	return(0);
}
/*
int cmd8_i2c_write7(int fd, int scl, int sda, int devid, int reg, int byte)
{
	if(i2c_write_byte(fd,scl,sda,1,0,(devid<<1)|0) != 0)
		return(-1);
	if(i2c_write_byte(fd,scl,sda,0, 0, reg) != 0)
		return(-1);
	if(i2c_write_byte(fd,scl,sda,0, 1, byte) != 0)
		return(-1);

	return(0);
}

int cmd8_i2c_read7(int fd, int scl, int sda, int devid, int reg)
{
	if(i2c_write_byte(fd,scl,sda,1,0,(devid<<1)|0) != 0)
		return(-1);
	if(i2c_write_byte(fd,scl,sda,0, 0, reg) != 0)
		return(-1);
	if(i2c_write_byte(fd,scl,sda,1, 0, (devid<<1)|1) != 0)
		return(-1);
	printf("%02x\n", i2c_read_byte(fd,scl,sda,1,1));
	return(0);
}*/

int cmd8_i2c(int fd, int scl, int sda, char *cmd)
{
	int x;
	const char *arg=NULL;
	unsigned char addr=0;

	arg=strtok(cmd, ",");

	while(arg)
	{
		const char *a=arg;
		char g;
		int addr=0;
		int start=0, stop=0, read=0, nack=0;
		int ret=0;
		int data;

		while(*a)
		{
			if((*a) == 'S')		start=1;
			else if((*a) == 'P')	stop=1;
			else if((*a) == 'A')	addr=1;
			else if((*a) == 'R')	read=1;
			else if((*a) == 'W')	read=0;
			else if((*a) == 'N')	nack=1;
			else if((*a) == 't')	started=0;
			else if((*a) == 'T')	started=1;
			else			break;
			a++;
		}

		if(addr)
		{
			sscanf(a, "%02x", &data);
			ret=i2c_write_byte(fd, scl, sda, start, stop, (data << 1) | read);
		}
		else if(read)
		{
			data=i2c_read_byte(fd, scl, sda, nack, stop);
			printf("%02x\n", data);
			ret=0;
		}
		else
		{
			sscanf(a, "%02x", &data);
			ret=i2c_write_byte(fd, scl, sda, start, stop, data);
		}

		if(ret != 0)
			fprintf(stderr, "'%s' failed\n", arg);

#if 0
		// 7-bit address r/w, i.e. A7R1c, A7W1c
		if(sscanf(arg, "A7R%02x%c", &addr, &g) == 1)
			i2c_write_byte(1, 0, (addr<<1) | 1);
		else if(sscanf(arg, "A7W%02x%c", &addr, &g) == 1)
			i2c_write_byte(1, 0, (addr<<1));
		else if(strcmp(arg, "r") == 0)
			printf("%02x\n", i2c_read_byte(0, 0));
		else if(strcmp(arg, "R") == 0)
			printf("%02x\n", i2c_read_byte(1, 1));
		else if(sscanf(arg, "w%02x%c", &addr, &g) == 1)
			i2c_write_byte(0, 0, addr);
		else if(sscanf(arg, "W%02x%c", &addr, &g) == 1)
			i2c_write_byte(1, 1, addr);
		else	fprintf(stderr, "Uknown i2c command '%s'\n", arg);
#endif

		arg=strtok(NULL, ",");
	}

	return(0);

#if 0
	fprintf(stderr, "Printing register map!\n");

	i2c_write_byte(1, 0, WRITE_7BIT(addr));
	i2c_write_byte(0, 0, 0);
	i2c_write_byte(1, 0, READ_7BIT(addr));
	for(x=0; x<0x39; x++)
	{
		printf("%02x] %d\n", x, i2c_read_byte(0, 0));
	}

	printf("%02x] %d\n", x, i2c_read_byte(1, 1));
	return(0);
#endif
}
 
void i2c_start_cond(int FD, int SCL, int SDA) {
  if (started) { // if started, do a restart cond
    // set SDA to 1
    pin_read(FD, SDA);
    I2C_delay();
    CLOCKSTRETCH();
    // Repeated start setup time, minimum 4.7us
    I2C_delay();
  }
  if (pin_read(FD, SDA) == 0) {
    arbitration_lost();
  }
  // SCL is high, set SDA from 1 to 0.
  pin_clear(FD, SDA);
  I2C_delay();
  pin_clear(FD, SCL);
  started = 1;
}
 
void i2c_stop_cond(int FD, int SCL, int SDA){
  // set SDA to 0
  pin_clear(FD, SDA);
  I2C_delay();
  // Clock stretching
  CLOCKSTRETCH();
  // Stop bit setup time, minimum 4us
  I2C_delay();
  // SCL is high, set SDA from 0 to 1
  if (pin_read(FD, SDA) == 0) {
    arbitration_lost();
  }
  I2C_delay();
  started = 0;
}
 
// Write a bit to I2C bus
void i2c_write_bit(int FD, int SCL, int SDA, int bit) {
  if (bit) {
    pin_read(FD, SDA);
  } else {
    pin_clear(FD, SDA);
  }
  I2C_delay();
  CLOCKSTRETCH();
  // SCL is high, now data is valid
  // If SDA is high, check that nobody else is driving SDA
  if (bit && pin_read(FD, SDA) == 0) {
    arbitration_lost();
  }
  I2C_delay();
  pin_clear(FD, SCL);
}
 
// Read a bit from I2C bus
int i2c_read_bit(int FD, int SCL, int SDA) {
  int bit;
  // Let the slave drive data
  pin_read(FD, SDA);
  I2C_delay();
  CLOCKSTRETCH();
  // SCL is high, now data is valid
  bit = pin_read(FD, SDA);
  I2C_delay();
  pin_clear(FD, SCL);
  return bit;
}
 
// Write a byte to I2C bus. Return 0 if ack by the slave.
int i2c_write_byte(int FD, int SCL, int SDA, int send_start,
                    int send_stop,
                    unsigned char byte) {

  INFO_PRINT("%s(%d, %d, 0x%02x)\n", __FUNCTION__, send_start, send_stop, byte);

  unsigned bit;
  int nack;
  if (send_start) {
    i2c_start_cond(FD, SCL, SDA);
  }
  for (bit = 0; bit < 8; bit++) {
    i2c_write_bit(FD, SCL, SDA, (byte & 0x80) != 0);
    byte <<= 1;
  }
  nack = i2c_read_bit(FD, SCL, SDA);
  if (send_stop) {
    i2c_stop_cond(FD, SCL, SDA);
  }
  return nack;
}
 
// Read a byte from I2C bus
unsigned char i2c_read_byte(int FD, int SCL, int SDA, int nack, int send_stop) {
  INFO_PRINT("%s(%d, %d)\n", __FUNCTION__, nack, send_stop);
  unsigned char byte = 0;
  unsigned bit;
  for (bit = 0; bit < 8; bit++) {
    byte = (byte << 1) | i2c_read_bit(FD, SCL, SDA);
  }
  i2c_write_bit(FD, SCL, SDA, nack);
  if (send_stop) {
    i2c_stop_cond(FD, SCL, SDA);
  }
  return byte;
}
